function AudioPlayer()
{
	var self = this;
	
	this.disabled = false;
	this.basePath = "";
	
	this.mp3Support = true;
	this.webAudioSupport = false;
	
	this.delayPlay = false;
	
	this.audioWrapper = null;
	
	this.locked = false;
	this.busy = false;
	
	this.startPlayTime = 0;
	
	this.onend = null;
	
	this.createNewAudio = function()
	{
		if(this.webAudioSupport)
		{
			var sound = AudioMixer.waContext.createBufferSource();
			sound.connect(AudioMixer.waContext.destination);
			return sound;
		}
		else
		{
			return document.createElement('audio');
		}
	};
	
	this.init = function(path, webAudioSupport)
	{
		this.webAudioSupport = webAudioSupport;
		this.basePath = path ? path : "";
		this.delayPlay = ("ontouchstart" in window);
		this.audioWrapper = this.createNewAudio();
		
		var test = document.createElement('audio');
		
		if(test.canPlayType) this.mp3Support = test.canPlayType('audio/mpeg') != "";
		else this.disabled = true;
		
		return !this.disabled;
	};
	
	this.play = function(file, loop)
	{
		if(this.disabled) return false;
		
		var url = this.basePath + "/" + file + (this.mp3Support ? ".mp3" : ".ogg");
		
		this.stop();
		
		this.audioWrapper = this.createNewAudio();
		this.audioWrapper.doLoop = (loop ? true : false);
		
		if(this.webAudioSupport)
		{
			var self = this;
			
			this.loadSound(url, function(buffer)
			{
				self.audioWrapper.buffer = buffer; 
				self.audioWrapper.noteOn ? self.audioWrapper.noteOn(0) : self.audioWrapper.start(0);
				self.startPlayTime = new Date().getTime();
				
				self.audioWrapper.loop = loop;
				
				self.waCheckInterval = setInterval(function()
				{
					if(!self.audioWrapper)
					{
						clearInterval(self.waCheckInterval);
						return;
					}
					
					if(self.audioWrapper.playbackState == self.audioWrapper.FINISHED_STATE)
					{
						self.controlPlay();
					}
				}, 100);
			});
		}
		else
		{
			console.log(file, AudioMixer.cache);
			
			if(AudioMixer.cache[file])
			{
				this.audioWrapper = AudioMixer.cache[file];
				this.audioWrapper.play();
			}
			else
			{
				this.audioWrapper.src = url;
				this.audioWrapper.type = (this.mp3Support ? "audio/mpeg" : "audio/ogg");
				this.audioWrapper.loop = false;
				this.audioWrapper.preload = "auto";
				this.audioWrapper.load();
				
				if(this.delayPlay) this.audioWrapper.addEventListener("canplay", this.readyToPlay, false);
				else this.audioWrapper.play();
			}
			
			this.audioWrapper.addEventListener("ended", this.controlPlay, false);
		}
		
		this.busy = true;
		this.startPlayTime = new Date().getTime();
	};
	
	this.loadSound = function(url, callback)
	{
		if(AudioMixer.buffer[url])
		{
			if(callback) callback(AudioMixer.buffer[url]);
			return;
		}
		
		var request = new XMLHttpRequest(); 
		request.open('GET', url, true); 
		request.responseType = 'arraybuffer'; 

		request.onload = function()
		{ 
			AudioMixer.waContext.decodeAudioData(this.response, function(buffer)
			{
				AudioMixer.buffer[url] = buffer;
				if(callback) callback(buffer);
			}); 
		} 
		request.send();
	}
	
	this.readyToPlay = function(e)
	{
		e.currentTarget.play();
	}
	
	this.stop = function()
	{
		this.busy = false;
		
		try
		{
			if(this.webAudioSupport)
			{
				this.audioWrapper.noteOff ? this.audioWrapper.noteOff(0) : this.audioWrapper.stop(0);
				this.audioWrapper = null;
			}
			else
			{
				this.audioWrapper.removeEventListener("canplay", this.readyToPlay);
				this.audioWrapper.pause();
				this.audioWrapper.currentTime = 0.0;
				this.audioWrapper = null;
			}
		}
		catch(e) {};
	};
	
	this.pause = function()
	{
		if(this.webAudioSupport)
		{
			if(self.audioWrapper) self.audioWrapper.disconnect();
		}
		else
		{
			this.audioWrapper.pause();
		}
	}
	
	this.resume = function()
	{
		if(this.webAudioSupport)
		{
			if(self.audioWrapper) self.audioWrapper.connect(AudioMixer.waContext.destination);
		}
		else
		{
			this.audioWrapper.play();
		}
	}
	
	this.controlPlay = function()
	{
		if(self.audioWrapper.doLoop)
		{
			if(!this.webAudioSupport)
			{
				self.audioWrapper.pause();
				self.audioWrapper.currentTime = 0.0;
				self.audioWrapper.play();
			}
		}
		else
		{
			self.busy = false;
			if(typeof self.onend == "function") self.onend();
			if(this.waCheckInterval)
			{
				clearInterval(this.waCheckInterval);
			}
		}
	}
	
	this.getPosition = function()
	{
		if(this.webAudioSupport)
		{
			if(!this.startPlayTime) return 0;
			
			var duration = this.getDuration();
			if(!duration) return 0;
			
			var position = ((new Date().getTime()) - this.startPlayTime)/1000;
				
			if(position <= duration) return position;
			
			if(!this.audioWrapper.doLoop) return duration;
			
			return position - Math.floor(position/duration)*duration;
		}
		else
		{
			return this.audioWrapper.currentTime ? this.audioWrapper.currentTime : 0;
		}
	}
	
	this.getDuration = function()
	{
		if(this.webAudioSupport)
		{
			return this.audioWrapper.buffer ? this.audioWrapper.buffer.duration : 0;
		}
		else
		{
			return this.audioWrapper.duration ? this.audioWrapper.duration : 0;
		}
	}
}

function AudioMixer(path, channelsCount)
{
	this.webAudioSupport = false;
	this.singleChannelMode = false;
	
	this.channels = [];
	
	this.init = function(path, channelsCount)
	{
		this.webAudioSupport = ("webkitAudioContext" in window);
		
		if(this.webAudioSupport)
		{
			AudioMixer.waContext = new webkitAudioContext();
			var buffer = AudioMixer.waContext.createBuffer(1, 1, 22050);
			sound = AudioMixer.waContext.createBufferSource();
			sound.buffer = buffer;
			sound.connect(AudioMixer.waContext.destination);
			sound.noteOn ? sound.noteOn(0) : sound.start(0);
		}
		
		if(!this.webAudioSupport && navigator.userAgent.toLowerCase().indexOf("mac") != -1)
		{
			this.singleChannelMode = true;
			channelsCount = 1;
		}
		
		this.path = path;
		
		this.channels = [];
		for(var i=0; i<channelsCount; i++)
		{
			this.channels[i] = new AudioPlayer();
			this.channels[i].init(path, this.webAudioSupport);
		}
		
		var hidden, visibilityChange;
		if (typeof document.hidden !== "undefined")
		{ 
			hidden = "hidden";
			visibilityChange = "visibilitychange";
		}
		else if (typeof document.mozHidden !== "undefined")
		{
			hidden = "mozHidden";
			visibilityChange = "mozvisibilitychange";
		}
		else if (typeof document.msHidden !== "undefined")
		{
			hidden = "msHidden";
			visibilityChange = "msvisibilitychange";
		}
		else if (typeof document.webkitHidden !== "undefined")
		{
			hidden = "webkitHidden";
			visibilityChange = "webkitvisibilitychange";
		}
		
		var self = this;
		function handleVisibilityChange()
		{
			if (document[hidden])
			{
				for(var i=0; i<self.channels.length; i++)
				{
					self.channels[i].pause();
				}
			}
			else
			{
				for(var i=0; i<self.channels.length; i++)
				{
					self.channels[i].resume();
				}
			}
		}
		
		document.addEventListener(visibilityChange, handleVisibilityChange, false);
	};
	
	this.play = function(file, loop, soft, channelID)
	{
		var cID = -1;
		
		if(typeof channelID == "number") cID = channelID;
		else cID = this.getFreeChannel(soft);
		
		if(cID >= 0 && cID < this.channels.length)
		{
			this.channels[cID].stop();
			this.channels[cID].play(file, loop);
		}
		
		return this.channels[cID];
	};
	
	this.stop = function(cID)
	{
		if(cID >= 0 && cID < this.channels.length) this.channels[cID].stop();
	};
	
	this.getFreeChannel = function(soft)
	{
		var cID = -1;
		var freeChannels = [];
		
		var maxID = -1;
		var max = -1;
		var t = 0;
		
		for(var i=0; i<this.channels.length; i++)
		{
			if(!this.channels[i].locked)
			{
				if(!this.channels[i].busy) freeChannels.push(i);
				else
				{
					t = new Date().getTime();
					t -= this.channels[i].startPlayTime;
					
					if(t > max)
					{
						max = t;
						maxID = i;
					}
				}
			}
		}
		
		if(freeChannels.length == 0)
		{
			if(!soft && max >= 0) cID = max;
		}
		else cID = freeChannels[0];
		
		return cID;
	};
	
	this.preload = function(path, sounds, callback)
	{
		var c, url, total = sounds.length, loaded = 0;
		
		AudioMixer.cache = {};
			
		for(var i=0; i<sounds.length; i++)
		{
			c = new AudioPlayer();
			url = path + "/" + sounds[i] + (c.mp3Support ? ".mp3" : ".ogg");
			c.init(path, this.webAudioSupport);
			
			if(this.webAudioSupport)
			{
				c.loadSound(url, function()
				{
					loaded++;
					if(loaded >= total) callback();
				});
			}
			else
			{
				c.audioWrapper.src = url;
				c.audioWrapper.type = (c.mp3Support ? "audio/mpeg" : "audio/ogg");
				c.audioWrapper.preload = "auto";
				c.audioWrapper.load();
				
				AudioMixer.cache[sounds[i]] = c.audioWrapper;
				
				c.audioWrapper.addEventListener("canplay", function(e)
				{
					loaded++;
					if(loaded >= total) callback();
				});
			}
		}
	}
	
	this.init(path, channelsCount);
}

AudioMixer.buffer = {};
AudioMixer.cache = {};
AudioMixer.waContext = null;