function box2DCreateWorld(gravity)
{
	var g, s;
	
	var worldAABB = new b2AABB();
	worldAABB.minVertex.Set(-1000, -1000);
	worldAABB.maxVertex.Set(1000, 1000);
	if(gravity) g = new b2Vec2(0, gravity);
	else g = new b2Vec2(0, 300);
	return new b2World(worldAABB, g, true);
}

function box2DCreateGround(world, restitution, y)
{
	var groundSd = new b2BoxDef();
	groundSd.extents.Set(10000, 50);
	groundSd.restitution = restitution ? restitution : 0.2;
	var groundBd = new b2BodyDef();
	groundBd.AddShape(groundSd);
	groundBd.position.Set(-500, y ? y : 480);
	return world.CreateBody(groundBd);
}

function box2DCreateCircle(world, x, y, radius, rotation, fixed, density, restitution, friction)
{
	if (typeof(fixed) == 'undefined') fixed = true;
	
	var ballSd = new b2CircleDef();
	if (!fixed)
	{
		ballSd.density = density ? density : 0.8;
		ballSd.restitution = restitution ? restitution : 0.1;
		ballSd.friction = friction ? friction : 3;
	}
	ballSd.radius = radius;
	var ballBd = new b2BodyDef();
	ballBd.rotation = rotation;
	ballBd.AddShape(ballSd);
	ballBd.position.Set(x, y);
	return world.CreateBody(ballBd);
}

function box2DCreateBox(world, x, y, width, height, rotation, fixed, density, restitution, friction)
{
	if (typeof(fixed) == 'undefined') fixed = true;
	
	var boxSd = new b2BoxDef();
	if (!fixed)
	{
		boxSd.density = density ? density : 1.0;
		boxSd.restitution = restitution ? restitution : 0.2;
		boxSd.friction = friction ? friction : 0.3;
	}
	boxSd.extents.Set(width, height);
	var boxBd = new b2BodyDef();
	boxBd.rotation = rotation;
	boxBd.AddShape(boxSd);
	boxBd.position.Set(x, y);
	return world.CreateBody(boxBd);
}

function box2DCreatePoly(world, x, y, points, rotation, fixed, density, restitution, friction)
{
	var polyBd = new b2BodyDef();
	polyBd.rotation = rotation;
	
	for(var n=0; n<points.length; n++)
	{
		var p = points[n];
		var polySd = new b2PolyDef();
		if (!fixed)
		{
			polySd.density = density ? density : 1.0;
			polySd.restitution = restitution ? restitution : 0.2;
			polySd.friction = friction ? friction : 0.3;
		}
		
		polySd.vertexCount = p.length;
		for (var i=0; i < p.length; i++)
		{
			polySd.vertices[i].Set(p[i][0], p[i][1]);
		}

		polyBd.AddShape(polySd);
	}
	
	polyBd.position.Set(x, y);
	return world.CreateBody(polyBd);
};

function box2DCreateRevoluteJoint(world, body1, body2, point, motor)
{
	var jointDef = new b2RevoluteJointDef();
	jointDef.body1 = body1;
	jointDef.body2 = body2;
	jointDef.anchorPoint = point;
	jointDef.collideConnected = true;
	
	if(motor)
	{
		jointDef.motorSpeed = motor;
		jointDef.motorTorque = 500000000;
		jointDef.enableMotor = true;
	}
	
	return world.CreateJoint(jointDef);
};

function box2DCreateDistanceJoint(world, body1, body2, point1, point2)
{
	var jointDef = new b2DistanceJointDef();
	jointDef.body1 = body1;
	jointDef.body2 = body2;
	jointDef.anchorPoint1 = point1;
	jointDef.anchorPoint2 = point2;
	jointDef.collideConnected = true;
	
	return world.CreateJoint(jointDef);
};

function box2DCreatePrismaticJoint(world, body1, body2, point, motor)
{
	var jointDef = new b2PrismaticJointDef();
	
	jointDef.anchorPoint.Set(point.x, point.y);
	
	jointDef.body1 = body1;
	jointDef.body2 = body2;
	
	jointDef.axis.Set(0.0, 1.0);
	
	if(motor)
	{
		jointDef.motorSpeed = motor;
		jointDef.motorForce = 100000.0;
		jointDef.enableMotor = true;
	}
	
	return world.CreateJoint(jointDef);
};

var Prototype = 
{
	Version: '1.6.0.2',

	Browser:
	{
		IE:     !!(window.attachEvent && !window.opera),
		Opera:  !!window.opera,
		WebKit: navigator.userAgent.indexOf('AppleWebKit/') > -1,
		Gecko:  navigator.userAgent.indexOf('Gecko') > -1 && navigator.userAgent.indexOf('KHTML') == -1,
		MobileSafari: !!navigator.userAgent.match(/Apple.*Mobile.*Safari/)
	},

	BrowserFeatures:
	{
		XPath: !!document.evaluate,
		ElementExtensions: !!window.HTMLElement,
		SpecificElementExtensions:
		document.createElement('div').__proto__ &&
		document.createElement('div').__proto__ !==
		document.createElement('form').__proto__
	},

	ScriptFragment: '<script[^>]*>([\\S\\s]*?)<\/script>',
	JSONFilter: /^\/\*-secure-([\s\S]*)\*\/\s*$/,

	emptyFunction: function() { },
	K: function(x) { return x }
};
if(Prototype.Browser.MobileSafari) Prototype.BrowserFeatures.SpecificElementExtensions = false;

var Class =
{
	create: function()
	{
		var parent = null, properties = $A(arguments);
		if (Object.isFunction(properties[0])) parent = properties.shift();

		function klass() { this.initialize.apply(this, arguments); }

		Object.extend(klass, Class.Methods);
		klass.superclass = parent;
		klass.subclasses = [];

		if (parent)
		{
			var subclass = function() { };
			subclass.prototype = parent.prototype;
			klass.prototype = new subclass;
			parent.subclasses.push(klass);
		}

		for (var i = 0; i < properties.length; i++)	klass.addMethods(properties[i]);
		if (!klass.prototype.initialize) klass.prototype.initialize = Prototype.emptyFunction;

		klass.prototype.constructor = klass;
		return klass;
	}
};

Class.Methods =
{
	addMethods: function(source)
	{
		var ancestor   = this.superclass && this.superclass.prototype;
		var properties = Object.keys(source);

		if (!Object.keys({ toString: true }).length) properties.push("toString", "valueOf");

		for (var i = 0, length = properties.length; i < length; i++)
		{
			var property = properties[i], value = source[property];
			if (ancestor && Object.isFunction(value) && value.argumentNames().first() == "$super")
			{
				var method = value;
				value = Object.extend((function(m) { return function() { return ancestor[m].apply(this, arguments) };})(property).wrap(method),
				{
					valueOf:  function() { return method },
					toString: function() { return method.toString() }
				});
			}
			this.prototype[property] = value;
		}
		return this;
	}
};

var Abstract = { };

Object.extend = function(destination, source)
{
	for (var property in source) destination[property] = source[property];
	return destination;
};

Object.extend(Object,
{
	inspect: function(object)
	{
		try
		{
			if (Object.isUndefined(object)) return 'undefined';
			if (object === null) return 'null';
			return object.inspect ? object.inspect() : String(object);
		}
		catch (e)
		{
			if (e instanceof RangeError) return '...';
			throw e;
		}
	},

	toJSON: function(object)
	{
		var type = typeof object;
		switch (type)
		{
			case 'undefined':
			case 'function':
			case 'unknown': return;
			case 'boolean': return object.toString();
		}

		if (object === null) return 'null';
		if (object.toJSON) return object.toJSON();
		if (Object.isElement(object)) return;

		var results = [];
		for (var property in object)
		{
			var value = Object.toJSON(object[property]);
			if (!Object.isUndefined(value))
			results.push(property.toJSON() + ': ' + value);
		}

		return '{' + results.join(', ') + '}';
	},

	toQueryString: function(object) {return $H(object).toQueryString();},
	toHTML: function(object) {return object && object.toHTML ? object.toHTML() : String.interpret(object);},

	keys: function(object)
	{
		var keys = [];
		for (var property in object)
		keys.push(property);
		return keys;
	},

	values: function(object)
	{
		var values = [];
		for (var property in object)
		values.push(object[property]);
		return values;
	},

	clone: function(object) {return Object.extend({ }, object);},
	isElement: function(object) {return object && object.nodeType == 1;},

	isArray: function(object)
	{
		return object != null && typeof object == "object" &&
		'splice' in object && 'join' in object;
	},

	isHash: function(object) {return object instanceof Hash;},
	isFunction: function(object) {return typeof object == "function";},
	isString: function(object) {return typeof object == "string";},
	isNumber: function(object) {return typeof object == "number";},
	isUndefined: function(object) {return typeof object == "undefined";}
});

Object.extend(Function.prototype,
{
	argumentNames: function()
	{
		var names = this.toString().match(/^[\s\(]*function[^(]*\((.*?)\)/)[1].split(",").invoke("strip");
		return names.length == 1 && !names[0] ? [] : names;
	},

	bind: function()
	{
		if (arguments.length < 2 && Object.isUndefined(arguments[0])) return this;
		var __method = this, args = $A(arguments), object = args.shift();
		return function() {return __method.apply(object, args.concat($A(arguments)));}
	},

	bindAsEventListener: function()
	{
		var __method = this, args = $A(arguments), object = args.shift();
		return function(event) {return __method.apply(object, [event || window.event].concat(args));}
	},

	curry: function()
	{
		if (!arguments.length) return this;
		var __method = this, args = $A(arguments);
		return function() {return __method.apply(this, args.concat($A(arguments)));}
	},

	delay: function()
	{
		var __method = this, args = $A(arguments), timeout = args.shift() * 1000;
		return window.setTimeout(function() {return __method.apply(__method, args);}, timeout);
	},

	wrap: function(wrapper)
	{
		var __method = this;
		return function() {return wrapper.apply(this, [__method.bind(this)].concat($A(arguments)));}
	},

	methodize: function()
	{
		if (this._methodized) return this._methodized;
		var __method = this;
		return this._methodized = function() {return __method.apply(null, [this].concat($A(arguments)));};
	}
});

Function.prototype.defer = Function.prototype.delay.curry(0.01);

Object.extend(String,
{
	interpret: function(value) {return value == null ? '' : String(value);},
	specialChar: {'\b': '\\b', '\t': '\\t', '\n': '\\n', '\f': '\\f', '\r': '\\r', '\\': '\\\\'}
});

Object.extend(String.prototype,
{
	gsub: function(pattern, replacement)
	{
		var result = '', source = this, match;
		replacement = arguments.callee.prepareReplacement(replacement);
		while (source.length > 0)
		{
			if (match = source.match(pattern))
			{
				result += source.slice(0, match.index);
				result += String.interpret(replacement(match));
				source  = source.slice(match.index + match[0].length);
			}
			else result += source, source = '';
		}
		return result;
	},

	sub: function(pattern, replacement, count)
	{
		replacement = this.gsub.prepareReplacement(replacement);
		count = Object.isUndefined(count) ? 1 : count;
		return this.gsub(pattern, function(match)
		{
			if (--count < 0) return match[0];
			return replacement(match);
		});
	},

	scan: function(pattern, iterator)
	{
		this.gsub(pattern, iterator);
		return String(this);
	},

	truncate: function(length, truncation)
	{
		length = length || 30;
		truncation = Object.isUndefined(truncation) ? '...' : truncation;
		return this.length > length ? this.slice(0, length - truncation.length) + truncation : String(this);
	},

	strip: function() {return this.replace(/^\s+/, '').replace(/\s+$/, '');},
	stripTags: function() {return this.replace(/<\/?[^>]+>/gi, '');},

	stripScripts: function() {return this.replace(new RegExp(Prototype.ScriptFragment, 'img'), '');},

	extractScripts: function()
	{
		var matchAll = new RegExp(Prototype.ScriptFragment, 'img');
		var matchOne = new RegExp(Prototype.ScriptFragment, 'im');
		return (this.match(matchAll) || []).map(function(scriptTag) {return (scriptTag.match(matchOne) || ['', ''])[1];});
	},

	evalScripts: function(){return this.extractScripts().map(function(script) { return eval(script) });},

	escapeHTML: function()
	{
		var self = arguments.callee;
		self.text.data = this;
		return self.div.innerHTML;
	},

	unescapeHTML: function()
	{
		var div = new Element('div');
		div.innerHTML = this.stripTags();
		return div.childNodes[0] ? (div.childNodes.length > 1 ?	$A(div.childNodes).inject('', function(memo, node) { return memo+node.nodeValue }) : div.childNodes[0].nodeValue) : '';
	},

	toQueryParams: function(separator)
	{
		var match = this.strip().match(/([^?#]*)(#.*)?$/);
		if (!match) return { };

		return match[1].split(separator || '&').inject({ }, function(hash, pair)
		{
			if ((pair = pair.split('='))[0])
			{
				var key = decodeURIComponent(pair.shift());
				var value = pair.length > 1 ? pair.join('=') : pair[0];
				if (value != undefined) value = decodeURIComponent(value);

				if (key in hash)
				{
					if (!Object.isArray(hash[key])) hash[key] = [hash[key]];
					hash[key].push(value);
				}
				else hash[key] = value;
			}
			return hash;
		});
	},

	toArray: function() {return this.split('');},
	succ: function(){return this.slice(0, this.length - 1) +	String.fromCharCode(this.charCodeAt(this.length - 1) + 1);},
	times: function(count) {return count < 1 ? '' : new Array(count + 1).join(this);},

	camelize: function()
	{
		var parts = this.split('-'), len = parts.length;
		if (len == 1) return parts[0];
		var camelized = this.charAt(0) == '-' ? parts[0].charAt(0).toUpperCase() + parts[0].substring(1) : parts[0];
		for (var i = 1; i < len; i++) camelized += parts[i].charAt(0).toUpperCase() + parts[i].substring(1);
			return camelized;
	},
	capitalize: function() {return this.charAt(0).toUpperCase() + this.substring(1).toLowerCase();},
	underscore: function() {return this.gsub(/::/, '/').gsub(/([A-Z]+)([A-Z][a-z])/,'#{1}_#{2}').gsub(/([a-z\d])([A-Z])/,'#{1}_#{2}').gsub(/-/,'_').toLowerCase();},
	dasherize: function() {return this.gsub(/_/,'-');},
	
	inspect: function(useDoubleQuotes)
	{
		var escapedString = this.gsub(/[\x00-\x1f\\]/, function(match)
		{
			var character = String.specialChar[match[0]];
			return character ? character : '\\u00' + match[0].charCodeAt().toPaddedString(2, 16);
		});
		if (useDoubleQuotes) return '"' + escapedString.replace(/"/g, '\\"') + '"';
		return "'" + escapedString.replace(/'/g, '\\\'') + "'";
	},

	toJSON: function() {return this.inspect(true);},
	unfilterJSON: function(filter) {return this.sub(filter || Prototype.JSONFilter, '#{1}');},

	isJSON: function()
	{
		var str = this;
		if (str.blank()) return false;
		str = this.replace(/\\./g, '@').replace(/"[^"\\\n\r]*"/g, '');
		return (/^[,:{}\[\]0-9.\-+Eaeflnr-u \n\r\t]*$/).test(str);
	},

	evalJSON: function(sanitize)
	{
		var json = this.unfilterJSON();
		try {if (!sanitize || json.isJSON()) return eval('(' + json + ')');} catch (e) { }
		throw new SyntaxError('Badly formed JSON string: ' + this.inspect());
	},

	include: function(pattern) {return this.indexOf(pattern) > -1;},
	startsWith: function(pattern) {return this.indexOf(pattern) === 0;},
	endsWith: function(pattern)
	{
		var d = this.length - pattern.length;
		return d >= 0 && this.lastIndexOf(pattern) === d;
	},

	empty: function() {return this == '';},
	blank: function() {return /^\s*$/.test(this);},
	interpolate: function(object, pattern) {return new Template(this, pattern).evaluate(object);}
});

if (Prototype.Browser.WebKit || Prototype.Browser.IE) Object.extend(String.prototype,
{
	escapeHTML: function() {return this.replace(/&/g,'&amp;').replace(/</g,'&lt;').replace(/>/g,'&gt;');},
	unescapeHTML: function() {return this.replace(/&amp;/g,'&').replace(/&lt;/g,'<').replace(/&gt;/g,'>');}
});

String.prototype.gsub.prepareReplacement = function(replacement)
{
	if (Object.isFunction(replacement)) return replacement;
	var template = new Template(replacement);
	return function(match) { return template.evaluate(match) };
};

String.prototype.parseQuery = String.prototype.toQueryParams;

Object.extend(String.prototype.escapeHTML,
{
	div:  document.createElement('div'),
	text: document.createTextNode('')
});

with (String.prototype.escapeHTML) div.appendChild(text);

var Template = Class.create(
{
	initialize: function(template, pattern)
	{
		this.template = template.toString();
		this.pattern = pattern || Template.Pattern;
	},

	evaluate: function(object)
	{
		if (Object.isFunction(object.toTemplateReplacements)) object = object.toTemplateReplacements();
		return this.template.gsub(this.pattern, function(match)
		{
			if (object == null) return '';

			var before = match[1] || '';
			if (before == '\\') return match[2];

			var ctx = object, expr = match[3];
			var pattern = /^([^.[]+|\[((?:.*?[^\\])?)\])(\.|\[|$)/;
			match = pattern.exec(expr);
			if (match == null) return before;

			while (match != null)
			{
				var comp = match[1].startsWith('[') ? match[2].gsub('\\\\]', ']') : match[1];
				ctx = ctx[comp];
				if (null == ctx || '' == match[3]) break;
				expr = expr.substring('[' == match[3] ? match[1].length : match[0].length);
				match = pattern.exec(expr);
			}

			return before + String.interpret(ctx);
		});
	}
});
Template.Pattern = /(^|.|\r|\n)(#\{(.*?)\})/;

var $break = { };

var Enumerable =
{
	each: function(iterator, context)
	{
		var index = 0;
		iterator = iterator.bind(context);
		try	{this._each(function(value) {iterator(value, index++);});}
		catch (e) {if (e != $break) throw e;}
		return this;
	},

	eachSlice: function(number, iterator, context)
	{
		iterator = iterator ? iterator.bind(context) : Prototype.K;
		var index = -number, slices = [], array = this.toArray();
		while ((index += number) < array.length)
		slices.push(array.slice(index, index+number));
		return slices.collect(iterator, context);
	},

	all: function(iterator, context)
	{
		iterator = iterator ? iterator.bind(context) : Prototype.K;
		var result = true;
		this.each(function(value, index)
		{
			result = result && !!iterator(value, index);
			if (!result) throw $break;
		});
		return result;
	},

	any: function(iterator, context)
	{
		iterator = iterator ? iterator.bind(context) : Prototype.K;
		var result = false;
		this.each(function(value, index)
		{
			if (result = !!iterator(value, index))
			throw $break;
		});
		return result;
	},

	collect: function(iterator, context)
	{
		iterator = iterator ? iterator.bind(context) : Prototype.K;
		var results = [];
		this.each(function(value, index) {results.push(iterator(value, index));});
		return results;
	},

	detect: function(iterator, context)
	{
		iterator = iterator.bind(context);
		var result;
		this.each(function(value, index)
		{
			if (iterator(value, index)) {result = value; throw $break;}
		});
		return result;
	},

	findAll: function(iterator, context)
	{
		iterator = iterator.bind(context);
		var results = [];
		this.each(function(value, index)
		{
			if (iterator(value, index))
			results.push(value);
		});
		return results;
	},

	grep: function(filter, iterator, context)
	{
		iterator = iterator ? iterator.bind(context) : Prototype.K;
		var results = [];
		if (Object.isString(filter))
		filter = new RegExp(filter);
		this.each(function(value, index)
		{
			if (filter.match(value))
			results.push(iterator(value, index));
		});
		return results;
	},

	include: function(object)
	{
		if (Object.isFunction(this.indexOf))
		if (this.indexOf(object) != -1) return true;
		var found = false;
		this.each(function(value)
		{
			if (value == object) {found = true; throw $break;}
		});
		return found;
	},

	inGroupsOf: function(number, fillWith)
	{
		fillWith = Object.isUndefined(fillWith) ? null : fillWith;
		return this.eachSlice(number, function(slice)
		{
			while(slice.length < number) slice.push(fillWith);
			return slice;
		});
	},

	inject: function(memo, iterator, context)
	{
		iterator = iterator.bind(context);
		this.each(function(value, index) {memo = iterator(memo, value, index);});
		return memo;
	},

	invoke: function(method)
	{
		var args = $A(arguments).slice(1);
		return this.map(function(value) {return value[method].apply(value, args);});
	},

	max: function(iterator, context)
	{
		iterator = iterator ? iterator.bind(context) : Prototype.K;
		var result;
		this.each(function(value, index)
		{
			value = iterator(value, index);
			if (result == null || value >= result) result = value;
		});
		return result;
	},

	min: function(iterator, context)
	{
		iterator = iterator ? iterator.bind(context) : Prototype.K;
		var result;
		this.each(function(value, index)
		{
			value = iterator(value, index);
			if (result == null || value < result) result = value;
		});
		return result;
	},

	partition: function(iterator, context)
	{
		iterator = iterator ? iterator.bind(context) : Prototype.K;
		var trues = [], falses = [];
		this.each(function(value, index)
		{
			(iterator(value, index) ? trues : falses).push(value);
		});
		return [trues, falses];
	},

	pluck: function(property)
	{
		var results = [];
		this.each(function(value) {results.push(value[property]);});
		return results;
	},

	reject: function(iterator, context)
	{
		iterator = iterator.bind(context);
		var results = [];
		this.each(function(value, index)
		{
			if (!iterator(value, index)) results.push(value);
		});
		return results;
	},

	sortBy: function(iterator, context)
	{
		iterator = iterator.bind(context);
		return this.map(function(value, index) {return {value: value, criteria: iterator(value, index)};}).sort(function(left, right)
		{
			var a = left.criteria, b = right.criteria;
			return a < b ? -1 : a > b ? 1 : 0;
		}).pluck('value');
	},

	toArray: function() {return this.map();},

	zip: function()
	{
		var iterator = Prototype.K, args = $A(arguments);
		if (Object.isFunction(args.last())) iterator = args.pop();

		var collections = [this].concat(args).map($A);
		return this.map(function(value, index) {return iterator(collections.pluck(index));});
	},

	size: function() {return this.toArray().length;},

	inspect: function() {return '#<Enumerable:' + this.toArray().inspect() + '>';}
};

Object.extend(Enumerable,
{
	map:     Enumerable.collect,
	find:    Enumerable.detect,
	select:  Enumerable.findAll,
	filter:  Enumerable.findAll,
	member:  Enumerable.include,
	entries: Enumerable.toArray,
	every:   Enumerable.all,
	some:    Enumerable.any
});


function $A(iterable)
{
	if (!iterable) return [];
	if (iterable.toArray) return iterable.toArray();
	var length = iterable.length || 0, results = new Array(length);
	while (length--) results[length] = iterable[length];
	return results;
}

if (Prototype.Browser.WebKit)
{
	$A = function(iterable)
	{
		if (!iterable) return [];
		if (!(Object.isFunction(iterable) && iterable == '[object NodeList]') && iterable.toArray) return iterable.toArray();
		var length = iterable.length || 0, results = new Array(length);
		while (length--) results[length] = iterable[length];
		return results;
	};
}

Array.from = $A;
Object.extend(Array.prototype, Enumerable);
if (!Array.prototype._reverse) Array.prototype._reverse = Array.prototype.reverse;

Object.extend(Array.prototype,
{
	_each: function(iterator) {	for (var i = 0, length = this.length; i < length; i++) iterator(this[i]); },
	clear: function() {this.length = 0; return this;},
	first: function() {return this[0];},
	last: function() {return this[this.length - 1];},
	compact: function()	{return this.select(function(value) {return value != null;});},

	flatten: function()
	{
		return this.inject([], function(array, value) {return array.concat(Object.isArray(value) ? value.flatten() : [value]); });
	},

	without: function()
	{
		var values = $A(arguments);
		return this.select(function(value) {return !values.include(value);});
	},

	reverse: function(inline) {return (inline !== false ? this : this.toArray())._reverse();},
	reduce: function() {return this.length > 1 ? this : this[0];},

	uniq: function(sorted)
	{
		return this.inject([], function(array, value, index)
		{
			if (0 == index || (sorted ? array.last() != value : !array.include(value)))	array.push(value);
			return array;
		});
	},

	intersect: function(array)
	{
		return this.uniq().findAll(function(item) { return array.detect(function(value) { return item === value }); });
	},

	clone: function() {return [].concat(this);},
	size: function() {return this.length;},
	inspect: function() {return '[' + this.map(Object.inspect).join(', ') + ']';},

	toJSON: function()
	{
		var results = [];
		this.each(function(object)
		{
			var value = Object.toJSON(object);
			if (!Object.isUndefined(value)) results.push(value);
		});
		return '[' + results.join(', ') + ']';
	}
});

if (Object.isFunction(Array.prototype.forEach)) Array.prototype._each = Array.prototype.forEach;

if (!Array.prototype.indexOf) Array.prototype.indexOf = function(item, i)
{
	i || (i = 0);
	var length = this.length;
	if (i < 0) i = length + i;
	for (; i < length; i++) if (this[i] === item) return i;
	return -1;
};

if (!Array.prototype.lastIndexOf) Array.prototype.lastIndexOf = function(item, i)
{
	i = isNaN(i) ? this.length : (i < 0 ? this.length + i : i) + 1;
	var n = this.slice(0, i).reverse().indexOf(item);
	return (n < 0) ? n : i - n - 1;
};
Array.prototype.toArray = Array.prototype.clone;